
package LFA2;

import LFA2.*;
import weka.core.*;
import org.apache.commons.math.distribution.*;
import org.apache.commons.math.random.*;
import java.io.*;

public class ConvertToCount{

    static double numStDev = 2;

    public static boolean needCovert(TableSpec table){
        int nAttrs = table.numAttributes();
        for(int i=0; i<nAttrs; i++){
            Attribute attr = table.attribute(i);
            String name = attr.name();
            if(name.matches("__LFAAvg\\(.+\\)__")){
                return true;
            }
        }
        return false;
    }

    // k: the maximum number of distinct values to be sampled
    public static void convert(String inFile, String outFile, int k) throws Exception{
        BufferedReader in = new BufferedReader(new FileReader(inFile));
        Instances dataset = new Instances(in);
        in.close();
        int nAttrs = dataset.numAttributes();
        int avgIndex = -1;
        int stdIndex = -1;
        int cntIndex = -1;
        String newName = null;
        for(int i=0; i<nAttrs; i++){
            Attribute attr = dataset.attribute(i);
            String name = attr.name();
            if(name.matches("__LFA.+__")){
                if(!attr.isNumeric()) throw new Exception("__LFA*__ must be numeric");
                if(name.equals("__LFACount__")){
                    if(cntIndex != -1) throw new Exception("only one __LFACount__ is allowed");
                    cntIndex = i;
                    continue;
                }
                name = name.replaceFirst("^__LFA", "");
                name = name.replaceFirst("__$", "");
                if(name.matches("^Avg\\(.+\\)$")){
                    name = name.replaceFirst("^Avg\\(","");
                    name = name.replaceFirst("\\)$", "");
                    if(avgIndex != -1) throw new Exception("only one __LFAAvg(...) is allowed");
                    avgIndex = i;
                    if(newName != null){
                        if(!newName.equals(name)) throw new Exception("Aggregation can be done on only one attribute ("+name+" != "+newName+")");
                    }else{
                        newName = name;
                    }
                }else if(name.matches("^StDev\\(.+\\)$")){
                    name = name.replaceFirst("^StDev\\(","");
                    name = name.replaceFirst("\\)$", "");
                    if(stdIndex != -1) throw new Exception("only one __LFAStDev(...) is allowed");
                    stdIndex = i;
                    if(newName != null){
                        if(!newName.equals(name)) throw new Exception("Aggregation can be done on only one attribute ("+name+" != "+newName+")");
                    }else{
                        newName = name;
                    }
                }else{
                    throw new Exception("unknown attribute name: "+attr.name()+
                                        "\nIt should be __LFAAvg(...)__, __LFAStDev(...)__, or __LFACount__");
                }
            }
        }
        if(avgIndex == -1){
            System.out.println("No AVG is found!! (No conversion)");
            return;
        }
        if(cntIndex == -1)
            throw new Exception("Aggregate views must contain __LFACount__");
        if(stdIndex != -1 && avgIndex+1 != stdIndex) 
            throw new Exception("Please order the attribute s.t. __LFAAvg(..)__, __LFAStDev(..)__");

        PrintStream out = new PrintStream(outFile);
        out.println("@relation "+dataset.relationName());
        for(int i=0; i<nAttrs; i++){
            Attribute attr = dataset.attribute(i);
            if(i==avgIndex){
                out.println("@attribute "+newName+" numeric");
            }else if(i==stdIndex){
                // do nothing
            }else{
                out.println(attr);
            }
        }
        out.println("@data");
        int nData = dataset.numInstances();
        RandomData rand = new RandomDataImpl();
        NormalDistribution normal = new NormalDistributionImpl();
        for(int i=0; i<nData; i++){
            Instance inst = dataset.instance(i);
            if(stdIndex == -1){
                out.println(inst);
                continue;
            }
            double[] tuple = inst.toDoubleArray();
            double count = tuple[cntIndex];
            double avg= tuple[avgIndex];
            double std= tuple[stdIndex];
            if(std == 0 || k <= 1){
                for(int a=0; a<nAttrs; a++){
                    if(a!=0 && a!=stdIndex) out.print(",");
                    if(a==avgIndex){
                        if(k>=0) out.print(avg+"");
                        else     out.print("0");
                    }else if(a==cntIndex){
                        if(k>0) out.print(count+"");
                        else    out.print("1");
                    }else if(a==stdIndex){
                        // do nothing
                    }else{
                        out.print(inst.toString(a));
                    }
                }
                out.println("");
                continue;
            }
            if(count <= k){
                for(int j=0; j<count; j++){
                    double value = rand.nextGaussian(avg,std);
                    for(int a=0; a<nAttrs; a++){
                        if(a!=0 && a!=stdIndex) out.print(",");
                        if(a==avgIndex){
                            out.print(value+"");
                        }else if(a==cntIndex){
                            out.print("1");
                        }else if(a==stdIndex){
                            // do nothing
                        }else{
                            out.print(inst.toString(a));
                        }
                    }
                    out.println("");
                }
            }else{
                double min = avg - (std*numStDev);
                double max = avg + (std*numStDev);
                double step= (max-min)/((double)k);
                normal.setMean(avg);
                normal.setStandardDeviation(std);
                double total = normal.cumulativeProbability(min,max);
                for(int j=0; j<k; j++){
                    double start = min + (j*step);
                    double end   = start + step;
                    double value = (start+end)/2.0;
                    double prob  = normal.cumulativeProbability(start,end);
                    for(int a=0; a<nAttrs; a++){
                        if(a!=0 && a!=stdIndex) out.print(",");
                        if(a==avgIndex){
                            out.print(value+"");
                        }else if(a==cntIndex){
                            out.print(""+(prob*count/total));
                        }else if(a==stdIndex){
                            // do nothing
                        }else{
                            out.print(inst.toString(a));
                        }
                    }
                    out.println("");
                }
            }
        }
    }

    public static void replicate(String inFile, String outFile, int k) throws Exception{
        BufferedReader in = new BufferedReader(new FileReader(inFile));
        PrintStream out = new PrintStream(outFile);
        String line = null;
        while((line=in.readLine()) != null){
            out.println(line);
            if("@data".equals(line.trim())) break;
        }
        while((line=in.readLine()) != null){
            for(int i=0; i<k; i++)
                out.println(line);
        }
    }

    public static void main(String[] argv) throws Exception{
        String usage = "java LFA2.ConvertToCount <input-file> <output-file> k\n";
        if(argv.length != 3){
            System.out.println(usage);
            System.exit(0);
        }
        int k = Integer.parseInt(argv[2]);
        convert(argv[0], argv[1], k);
    }

}
