package LFA2;

import LFA2.*;
import weka.core.*;
import java.io.*;

public class Unaggregate{

    static void convert(String inFile, String outFile) throws Exception{
        BufferedReader in = new BufferedReader(new FileReader(inFile));
        Instances dataset = new Instances(in);
        in.close();
        int nAttrs = dataset.numAttributes();
        int avgIndex = -1;
        int stdIndex = -1;
        int cntIndex = -1;
        String newName = null;
        for(int i=0; i<nAttrs; i++){
            Attribute attr = dataset.attribute(i);
            String name = attr.name();
            if(name.matches("__LFA.+__")){
                if(!attr.isNumeric()) throw new Exception("__LFA*__ must be numeric");
                if(name.equals("__LFACount__")){
                    if(cntIndex != -1) throw new Exception("only one __LFACount__ is allowed");
                    cntIndex = i;
                    continue;
                }
                name = name.replaceFirst("^__LFA", "");
                name = name.replaceFirst("__$", "");
                if(name.matches("^Avg\\(.+\\)$")){
                    name = name.replaceFirst("^Avg\\(","");
                    name = name.replaceFirst("\\)$", "");
                    if(avgIndex != -1) throw new Exception("only one __LFAAvg(...) is allowed");
                    avgIndex = i;
                    if(newName != null){
                        if(!newName.equals(name)) throw new Exception("Aggregation can be done on only one attribute ("+name+" != "+newName+")");
                    }else{
                        newName = name;
                    }
                }else if(name.matches("^StDev\\(.+\\)$")){
                    name = name.replaceFirst("^StDev\\(","");
                    name = name.replaceFirst("\\)$", "");
                    if(stdIndex != -1) throw new Exception("only one __LFAStDev(...) is allowed");
                    stdIndex = i;
                    if(newName != null){
                        if(!newName.equals(name)) throw new Exception("Aggregation can be done on only one attribute ("+name+" != "+newName+")");
                    }else{
                        newName = name;
                    }
                }else{
                    throw new Exception("unknown attribute name: "+attr.name()+
                                        "\nIt should be __LFAAvg(...)__, __LFAStDev(...)__, or __LFACount__");
                }
            }
        }
        if(avgIndex != -1 || stdIndex != -1){
            System.out.println("AVG or StDev is found!! (No conversion)");
            return;
        }
        if(cntIndex == -1)
            throw new Exception("Aggregate views must contain __LFACount__");

        PrintStream out = new PrintStream(new FileOutputStream(outFile));
        out.println("@relation "+dataset.relationName());
        for(int i=0; i<nAttrs; i++){
            Attribute attr = dataset.attribute(i);
            if(i==cntIndex){
                // do nothing
            }else{
                out.println(attr);
            }
        }
        out.println("@data");
        int nData = dataset.numInstances();
        for(int i=0; i<nData; i++){
            Instance inst = dataset.instance(i);
            double[] tuple = inst.toDoubleArray();
            double count = tuple[cntIndex];
            for(int j=0; j<count; j++){
                for(int a=0; a<nAttrs; a++){
                    if(a!=0 && a!=cntIndex) out.print(",");
                    if(a==cntIndex){
                        // do nothing
                    }else{
                        out.print(inst.toString(a));
                    }
                }
                out.println("");
            }
        }
    }

    public static void main(String[] argv) throws Exception{
        String usage = "java LFA2.Unaggregate <input-file> <output-file>\n";
        if(argv.length != 2){
            System.out.println(usage);
            System.exit(0);
        }
        convert(argv[0], argv[1]);
    }
}
